/* Update.h:							-*- C++ -*-

   Copyright (C) 2002-2006 Christian Schenk

   This file is part of the MiKTeX Update Wizard.

   The MiKTeX Update Wizard is free software; you can redistribute it
   and/or modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2, or
   (at your option) any later version.

   The MiKTeX Update Wizard is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with the MiKTeX Update Wizard; if not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA. */

#pragma once

#include "CdPage.h"
#include "FileCopyPage.h"
#include "FinishPage.h"
#include "LocalPage.h"
#include "PackageListPage.h"
#include "RemotePage.h"
#include "TypePage.h"
#include "Update.h"
#include "WelcomePage.h"

class UpdateWizard : public CPropertySheet
{
private:
  DECLARE_DYNAMIC(UpdateWizard);
  
protected:
  DECLARE_MESSAGE_MAP();
  
public:
  UpdateWizard ();
  
protected:
  virtual
  BOOL
  OnInitDialog ();

public:
  void
  SetUpdateList (/*[in]*/ const vector<string> & toBeUpdated)
  {
    this->toBeUpdated = toBeUpdated;
  }

public:
  const vector<string> &
  GetUpdateList ()
    const
  {
    return (toBeUpdated);
  }

public:
  void
  SetErrorFlag ()
  {
    error = true;
  }

public:
  void
  ClearErrorFlag ()
  {
    error = false;
  }

public:
  bool
  GetErrorFlag ()
    const
  {
    return (error);
  }

public:
  void
  SetCancelFlag ()
  {
    EnableCancelButton (false);
    cancelled = true;
  }

public:
  bool
  GetCancelFlag ()
    const
  {
    return (cancelled);
  }

public:
  void
  EnableCloseButton ()
  {
    SetWizardButtons (PSWIZB_FINISH);
    EnableCancelButton (false);
    SetFinishText (T_("Close"));
  }

public:
  void
  SetRandomRepositoryFlag (/*[in]*/ bool f)
  {
    randomRepository = "";
    randomRepositoryFlag = f;
  }

public:
  void
  SetRepositoryType (/*[in]*/ RepositoryType repositoryType)
  {
    this->repositoryType = repositoryType;
  }

public:
  RepositoryType
  GetRepositoryType ()
    const
  {
    return (repositoryType);
  }

public:
  void
  SetCameFrom (/*[in]*/ int pageId)
  {
    cameFrom = pageId;
  }

public:
  int
  GetCameFrom ()
  {
    int ret = cameFrom;
    cameFrom = 0;
    return (ret);
  }

public:
  string
  GetRepository ();

public:
  void
  ReportError (/*[in]*/ const exception & e);

public:
  void
  ReportError (/*[in]*/ const MiKTeXException & e);

private:
  void
  EnableCancelButton (/*[in]*/ bool enable);

private:
  CBitmap watermarkBitmap;
  
private:
  CBitmap headerBitmap;
  
private:
  WelcomePage welcomePage;
  
private:
  PackageListPage packageListPage;
  
private:
  FileCopyPage fileCopyPage;
  
private:
  TypePage typePage;
  
private:
  LocalPage localPage;
  
private:
  RemotePage remotePage;

private:
  CdPage cdPage;
  
private:
  FinishPage finishPage;
  
private:
  vector<string> toBeUpdated;

private:
  bool error;

private:
  bool cancelled;

private:
  bool randomRepositoryFlag;

private:
  RepositoryType repositoryType;

private:
  string randomRepository;

private:
  int cameFrom;
};
